---
title: 'Datasets'
date: date:2018-11-28T15:14:39+10:00
weight: 5
---

<!--
#################################################
### THIS FILE WAS AUTOGENERATED! DO NOT EDIT! ###
#################################################
# file to edit: nbs/data.dataset.ipynb
# command to build the docs after a change: nbdev_build_docs
-->
<div class="container" id="notebook-container">
        
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>A dataset is a central item in the pod that organizes your project data and label annotations. To facilitate using <a href="/component-architectures/plugins/datasets#Dataset"><code>Dataset</code></a> items in your datascience workflow, the <a href="/component-architectures/plugins/datasets#Dataset"><code>Dataset</code></a> class contains methods to convert the data to a popular datascience format, or save a dataset to disk.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h2 id="Dataset" class="doc_header"><code>class</code> <code>Dataset</code></h2><blockquote><p><code>Dataset</code>(<strong>**<code>kwargs</code></strong>) :: <a href="/component-architectures/plugins/datasets#Dataset"><code>Dataset</code></a></p>
</blockquote>
<pre><code class="nbdev-code chroma">The main Dataset class</code></pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h4 id="Dataset.to" class="doc_header"><code>Dataset.to</code></h4><blockquote><p><code>Dataset.to</code>(<strong><code>dtype</code></strong>:<code>str</code>, <strong><code>columns</code></strong>:<code>List</code>[<code>str</code>], <strong><code>filter_missing</code></strong>:<code>bool</code>=<em><code>True</code></em>)</p>
</blockquote>
<pre><code class="nbdev-code chroma">Converts Dataset to a different format.
Available formats:
list: a 2-dimensional list, containing one dataset entry per row
dict: a list of dicts, where each dict contains {column: value} for each column
pd: a Pandas dataframe

Args:
    dtype (str): Datatype of the returned dataset
    columns (List[str]): Column names of the dataset
    filter_missing (bool, optional): If true, all rows that contain `None` values are omitted.
        Defaults to True.
Returns:
    Any: Dataset formatted according to `dtype`</code></pre>
</div>
</div>
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h4 id="Dataset.save" class="doc_header"><code>Dataset.save</code></h4><blockquote><p><code>Dataset.save</code>(<strong><code>path</code></strong>:<code>Union</code>[<code>Path</code>, <code>str</code>], <strong><code>columns</code></strong>:<code>List</code>[<code>str</code>], <strong><code>filter_missing</code></strong>:<code>bool</code>=<em><code>True</code></em>)</p>
</blockquote>
<pre><code class="nbdev-code chroma">Save dataset to CSV.</code></pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Usage">Usage</h2><p>To convert the data in the pod to a different format, <a href="/component-architectures/plugins/datasets#Dataset"><code>Dataset</code></a> implements the <a href="/component-architectures/plugins/datasets#Dataset.to"><code>Dataset.to</code></a> method. In the <code>columns</code> argument, you can define which features will be included in your dataset. A <code>column</code> is either a property of an entry in the dataset, or a property of an item connected to an entry in the dataset.</p>
<p>The Pod uses the following schema for Dataset items. Note that the <code>DatasetEntry</code> item is always included, and the actual data can be found by traversing the <code>entry.data</code> Edge.</p>
<p><img src="/img/pymemri/dataset-diagram.png" alt="dataset schema"></p>
<p>Now for example, if a dataset is a set of <code>Message</code> items, and the content has to be included as column, <code>data.content</code> would be the column name. If the name of the <code>sender</code> of a message has to be included, <code>data.sender.handle</code> would be a valid column name.</p>
<p>The following example retrieves an example dataset of <code>Message</code> items, and formats them to a Pandas dataframe:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span> <span class="o">=</span> <span class="n">PodClient</span><span class="p">()</span>
<span class="n">client</span><span class="o">.</span><span class="n">add_to_schema</span><span class="p">(</span><span class="n">Dataset</span><span class="p">,</span> <span class="n">DatasetEntry</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>True</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">dataset</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">get_dataset</span><span class="p">(</span><span class="s2">&quot;example-dataset&quot;</span><span class="p">)</span>
<span class="n">columns</span> <span class="o">=</span> <span class="p">[</span><span class="s2">&quot;data.content&quot;</span><span class="p">,</span> <span class="s2">&quot;data.sender.handle&quot;</span><span class="p">,</span> <span class="s2">&quot;annotation.name&quot;</span><span class="p">]</span>
<span class="n">dataframe</span> <span class="o">=</span> <span class="n">dataset</span><span class="o">.</span><span class="n">to</span><span class="p">(</span><span class="s2">&quot;pd&quot;</span><span class="p">,</span> <span class="n">columns</span><span class="o">=</span><span class="n">columns</span><span class="p">)</span>
<span class="n">dataframe</span><span class="o">.</span><span class="n">head</span><span class="p">()</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_html rendered_html output_subarea output_execute_result">
<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }
    .dataframe tbody tr th {
        vertical-align: top;
    }
    .dataframe thead th {
        text-align: right;
    }
</style>
<table border="1" class="dataframe">
  <thead>
    <tr style="text-align: right;">
      <th></th>
      <th>id</th>
      <th>data.content</th>
      <th>data.sender.handle</th>
      <th>annotation.name</th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th>0</th>
      <td>371dbdda6d854434b256e4d826cbeec0</td>
      <td>content_0</td>
      <td>account_0</td>
      <td>label_0</td>
    </tr>
    <tr>
      <th>1</th>
      <td>989a77c9bd7c4e7ba3a0d78656c77b06</td>
      <td>content_1</td>
      <td>account_1</td>
      <td>label_1</td>
    </tr>
    <tr>
      <th>2</th>
      <td>8af555d92cf8406a9e7ad99d9c168360</td>
      <td>content_2</td>
      <td>account_2</td>
      <td>label_2</td>
    </tr>
    <tr>
      <th>3</th>
      <td>67318a855d844e7da69beebd408871fd</td>
      <td>content_3</td>
      <td>account_3</td>
      <td>label_3</td>
    </tr>
    <tr>
      <th>4</th>
      <td>e7fd9c347dc941c994eb5a56a63354e0</td>
      <td>content_4</td>
      <td>account_4</td>
      <td>label_4</td>
    </tr>
  </tbody>
</table>
</div>
</div>
</div>
</div>
</div>
</div>
    
</div>
 
