---
title: 'Plugin Configuration'
date: date:2018-11-28T15:14:39+10:00
weight: 4
---

<!--
#################################################
### THIS FILE WAS AUTOGENERATED! DO NOT EDIT! ###
#################################################
# file to edit: nbs/template.config.ipynb
# command to build the docs after a change: nbdev_build_docs
-->
<div class="container" id="notebook-container">
        
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Often, plugins require some configuration before running. For example you might want to run plugin on data from a specific service, or run a specific version of your machine learning model. To configure your plugin when running it from the front-end, plugins require a <code>config.json</code> in the root of the plugin repository. This file contains a declarative definition, which our front-end app uses to display a configuration screen.</p>
<p>This module contains utilities to directly generate this config file from the plugin definition, by inferring all arguments from the plugin <code>__init__</code> method. All configuration fields are textboxes by default, which can be changed to different types in the future. For a full example, see our guide on <a href="https://memri.docs.memri.io/docs.memri.io/guides/build_and_deploy_your_model/">building plugins</a>.</p>
<p>Usage:</p>
<pre><code class="nbdev-code chroma">create_plugin_config</code></pre>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h4 id="create_config" class="doc_header"><code>create_config</code></h4><blockquote><p><code>create_config</code>(<strong><code>plugin_cls</code></strong>:<code>type</code>)</p>
</blockquote>
<pre><code class="nbdev-code chroma">Returns a declarative plugin configuration, inferred from the `__init__` method signature of `plugin_cls`.
This function is used internally by the `create_plugin_config` CLI. For general use, use the CLI instead.
Arguments that start with `_`, untyped arguments or arguments that are not in `ALLOWED_TYPES` are skipped.
Args:
    plugin_cls (type): A plugin class, inherited from PluginBase.
Returns:
    List[dict]: A declarative configuration definition as list of dictionaries.</code></pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h4 id="create_plugin_config" class="doc_header"><code>create_plugin_config</code></h4><blockquote><p><code>create_plugin_config</code>(<strong><code>metadata</code></strong>:"metadata.json of the plugin"=<em><code>'./metadata.json'</code></em>, <strong><code>tgt_file</code></strong>:"Filename of config file"=<em><code>'config.json'</code></em>, <strong><code>schema_file</code></strong>:"Filename of exported plugin schema"=<em><code>'schema.json'</code></em>)</p>
</blockquote>
<pre><code class="nbdev-code chroma">Creates a plugin configuration definition from the arguments of your plugin class.
Configuration arguments are inferred from the arguments of your plugin `__init__` method.
Arguments that start with `_`, untyped arguments or arguments that are not in `ALLOWED_TYPES` are skipped.
All generated fields are "textbox" by default, in the future our front-end will support more
types of fields.
Args:
    metadata (Param, optional): Location of the "metadata.json" file,
        Defaults to "./metadata.json"
    tgt_file (Param, optional): File the config definition is saved to.
        Defaults to "config.json".</code></pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h3 id="Example">Example</h3><p>As example, we create a test plugin with various configuration arguments. The <a href="/component-architectures/plugins/plugin_config#create_config"><code>create_config</code></a> method is called on the plugin, and generates a declarative configuration field for each plugin argument. Note that arguments that start with a <code>_</code>, and untyped arguments are skipped in the annotation.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="k">class</span> <span class="nc">MyPlugin</span><span class="p">(</span><span class="n">PluginBase</span><span class="p">):</span>
    <span class="k">def</span> <span class="fm">__init__</span><span class="p">(</span>
        <span class="bp">self</span><span class="p">,</span>
        <span class="n">my_arg</span><span class="p">:</span> <span class="nb">str</span><span class="p">,</span>
        <span class="n">my_str</span><span class="p">:</span> <span class="nb">str</span> <span class="o">=</span> <span class="kc">None</span><span class="p">,</span>
        <span class="n">my_int</span><span class="p">:</span> <span class="nb">int</span> <span class="o">=</span> <span class="kc">None</span><span class="p">,</span>
        <span class="n">my_float</span><span class="p">:</span> <span class="nb">float</span> <span class="o">=</span> <span class="kc">None</span><span class="p">,</span>
        <span class="n">unannotated</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span>
        <span class="n">_private</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span>
        <span class="o">**</span><span class="n">kwargs</span>
    <span class="p">):</span>
        <span class="nb">super</span><span class="p">()</span><span class="o">.</span><span class="fm">__init__</span><span class="p">(</span><span class="o">**</span><span class="n">kwargs</span><span class="p">)</span>
    <span class="k">def</span> <span class="nf">run</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
        <span class="k">pass</span>
    <span class="k">def</span> <span class="nf">add_to_schema</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
        <span class="k">pass</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">config</span> <span class="o">=</span> <span class="n">create_config</span><span class="p">(</span><span class="n">MyPlugin</span><span class="p">)</span>
<span class="k">for</span> <span class="n">c</span> <span class="ow">in</span> <span class="n">config</span><span class="p">:</span>
    <span class="nb">print</span><span class="p">(</span><span class="sa">f</span><span class="s1">&#39;</span><span class="si">{</span><span class="n">c</span><span class="p">[</span><span class="s2">&quot;display&quot;</span><span class="p">]</span><span class="si">}</span><span class="s1"> (</span><span class="si">{</span><span class="n">c</span><span class="p">[</span><span class="s2">&quot;data_type&quot;</span><span class="p">]</span><span class="si">}</span><span class="s1">)&#39;</span><span class="p">)</span> 
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>Skipping unannotated parameter `unannotated`
My Arg (Text)
My Str (Text)
My Int (Integer)
My Float (Real)
</pre>
</div>
</div>
</div>
</div>
</div>
    
</div>
 
