---
title: 'The Pod Client'
date: date:2018-11-28T15:14:39+10:00
weight: 2
---

<!--
#################################################
### THIS FILE WAS AUTOGENERATED! DO NOT EDIT! ###
#################################################
# file to edit: nbs/pod.client.ipynb
# command to build the docs after a change: nbdev_build_docs
-->
<div class="container" id="notebook-container">
        
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h2 id="PodClient" class="doc_header"><code>class</code> <code>PodClient</code></h2><blockquote><p><code>PodClient</code>(<strong><code>url</code></strong>=<em><code>'http://localhost:3030'</code></em>, <strong><code>version</code></strong>=<em><code>'v4'</code></em>, <strong><code>database_key</code></strong>=<em><code>None</code></em>, <strong><code>owner_key</code></strong>=<em><code>None</code></em>, <strong><code>auth_json</code></strong>=<em><code>None</code></em>, <strong><code>register_base_schema</code></strong>=<em><code>True</code></em>, <strong><code>verbose</code></strong>=<em><code>False</code></em>, <strong><code>default_priority</code></strong>=<em><code>&lt;Priority.local: 'local'&gt;</code></em>)</p>
</blockquote>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Pymemri communicates with the pod via the <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a>. The PodClient requires you to provide a <a href="https://gitlab.memri.io/memri/pod/-/blob/dev/docs/HTTP_API.md#user-content-api-authentication-credentials">database key</a> and an <a href="https://gitlab.memri.io/memri/pod/-/blob/dev/docs/HTTP_API.md#user-content-api-authentication-credentials">owner key</a>. During development, you don't have to worry about these keys, you can just omit the keys when initializing the <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a>, which creates a new user by defining random keys.</p>
<p>If you want to use the same keys for different <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a> instances, you can store a random key pair locally with the <a href="/component-architectures/plugins/pluginbase#store_keys"><code>store_keys</code></a> CLI, and create a new client with <a href="/component-architectures/plugins/pod_client#PodClient.from_local_keys("><code>PodClient.from_local_keys()</code></a>). When you are using the app, setting the keys in the pod, and passing them when calling a plugin is handled for you by the app itself.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span> <span class="o">=</span> <span class="n">PodClient</span><span class="p">()</span>
<span class="n">client</span><span class="o">.</span><span class="n">registered_classes</span><span class="p">[</span><span class="s2">&quot;Photo&quot;</span><span class="p">]</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>pymemri.data.photo.Photo</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Creating-Items-and-Edges">Creating Items and Edges</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Now that we have access to the pod, we can create items here and upload them to the pod. All items are defined in the schema of the pod. To create an item in the pod, you have to add the schema first. Schemas can be added as follows</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="kn">from</span> <span class="nn">pymemri.data.schema</span> <span class="kn">import</span> <span class="n">EmailMessage</span><span class="p">,</span> <span class="n">Address</span><span class="p">,</span> <span class="n">PhoneNumber</span>
<span class="n">succes</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">add_to_schema</span><span class="p">(</span><span class="n">EmailMessage</span><span class="p">,</span> <span class="n">Address</span><span class="p">,</span> <span class="n">PhoneNumber</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>We can now create an item with one of the above item definitions. As a side-effect, our item will be assigned an id.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">email_item</span> <span class="o">=</span> <span class="n">EmailMessage</span><span class="o">.</span><span class="n">from_data</span><span class="p">(</span><span class="n">content</span><span class="o">=</span><span class="s2">&quot;example content field&quot;</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">email_item</span><span class="p">)</span>
<span class="nb">print</span><span class="p">(</span><span class="n">email_item</span><span class="o">.</span><span class="n">id</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>29c0b7a3f8754d02a7d0e641523c25bb
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>The types of items in the pod are not limited to definitions to the pymemri schema. We can easily define our own types, or overwrite existing item definitions with the same <code>add_to_schema</code> method.</p>
<p>Note that all keyword arguments need to be added to the <code>properties</code> class variable to let the pod know what the properties of our item are. Additionally, properties in the Pod are statically typed, and have to be inferred from type the annotations of our <code>__init__</code> method.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_markdown rendered_html output_subarea ">
<h2 id="Dog" class="doc_header"><code>class</code> <code>Dog</code></h2><blockquote><p><code>Dog</code>(<strong><code>name</code></strong>:<code>str</code>=<em><code>None</code></em>, <strong><code>age</code></strong>:<code>int</code>=<em><code>None</code></em>, <strong><code>bites</code></strong>:<code>bool</code>=<em><code>False</code></em>, <strong><code>weight</code></strong>:<code>float</code>=<em><code>None</code></em>, <strong>**<code>kwargs</code></strong>) :: <a href="/component-architectures/plugins/itembase#Item"><code>Item</code></a></p>
</blockquote>
<pre><code class="nbdev-code chroma">Item is the baseclass for all of the data classes.</code></pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span><span class="o">.</span><span class="n">add_to_schema</span><span class="p">(</span><span class="n">Dog</span><span class="p">)</span>
<span class="n">dog2</span> <span class="o">=</span> <span class="n">Dog</span><span class="p">(</span><span class="n">name</span><span class="o">=</span><span class="s2">&quot;bob&quot;</span><span class="p">,</span> <span class="n">age</span><span class="o">=</span><span class="mi">3</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mf">33.2</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">dog2</span><span class="p">);</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>We can connect items using edges. Let's create another item, a person, and connect the email and the person.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item</span> <span class="o">=</span> <span class="n">Person</span><span class="o">.</span><span class="n">from_data</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Alice&quot;</span><span class="p">,</span> <span class="n">lastName</span><span class="o">=</span><span class="s2">&quot;X&quot;</span><span class="p">)</span>
<span class="n">succes</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">add_to_schema</span><span class="p">(</span><span class="n">person_item</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span><span class="o">.</span><span class="n">local_db</span><span class="o">.</span><span class="n">contains</span><span class="p">(</span><span class="n">email_item</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>True</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span><span class="o">.</span><span class="n">local_db</span><span class="o">.</span><span class="n">nodes</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>{&#39;29c0b7a3f8754d02a7d0e641523c25bb&#39;: EmailMessage (#29c0b7a3f8754d02a7d0e641523c25bb),
 &#39;93d7acb23bdc4902bd8e4c44d49a3953&#39;: Dog (#93d7acb23bdc4902bd8e4c44d49a3953)}</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item</span> <span class="o">=</span> <span class="n">Person</span><span class="o">.</span><span class="n">from_data</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Alice&quot;</span><span class="p">,</span> <span class="n">lastName</span><span class="o">=</span><span class="s2">&quot;X&quot;</span><span class="p">)</span>
<span class="n">item_succes</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">person_item</span><span class="p">)</span>
<span class="n">edge</span> <span class="o">=</span> <span class="n">Edge</span><span class="p">(</span><span class="n">email_item</span><span class="p">,</span> <span class="n">person_item</span><span class="p">,</span> <span class="s2">&quot;sender&quot;</span><span class="p">)</span>
<span class="n">edge_succes</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">create_edge</span><span class="p">(</span><span class="n">edge</span><span class="p">)</span>
<span class="nb">print</span><span class="p">(</span><span class="n">client</span><span class="o">.</span><span class="n">get_edges</span><span class="p">(</span><span class="n">email_item</span><span class="o">.</span><span class="n">id</span><span class="p">))</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>[{&#39;item&#39;: Person (#5594a26fa6494f68a939eb610e34e7fe), &#39;name&#39;: &#39;sender&#39;}]
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>If we use the normal <code>client.get</code> (without <code>expanded=False</code>), we also get items directly connected to the Item.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">email_from_db</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">email_item</span><span class="o">.</span><span class="n">id</span><span class="p">)</span> 
<span class="nb">print</span><span class="p">(</span><span class="n">email_from_db</span><span class="o">.</span><span class="n">sender</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>[Person (#5594a26fa6494f68a939eb610e34e7fe)]
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h1 id="Fetching-and-updating-Items">Fetching and updating Items</h1>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Normal-Items">Normal Items</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>We can use the client to fetch data from the database. This is in particular useful for indexers, which often use data in the database as input for their models. The simplest form  of querying the database is by querying items in the pod by their id (unique identifier).</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item</span> <span class="o">=</span> <span class="n">Person</span><span class="o">.</span><span class="n">from_data</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Alice&quot;</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">person_item</span><span class="p">)</span>
<span class="c1"># Retrieve person from Pod</span>
<span class="n">person_from_db</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">person_item</span><span class="o">.</span><span class="n">id</span><span class="p">,</span> <span class="n">expanded</span><span class="o">=</span><span class="kc">False</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item</span><span class="o">.</span><span class="n">to_json</span><span class="p">(),</span> <span class="n">person_item</span><span class="o">.</span><span class="n">_in_pod</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>({&#39;id&#39;: &#39;1bdefab867c74f22ad156eebac482c1c&#39;,
  &#39;dateCreated&#39;: 1644321461975,
  &#39;dateModified&#39;: 1644321461975,
  &#39;dateServerModified&#39;: 1644321461975,
  &#39;deleted&#39;: False,
  &#39;firstName&#39;: &#39;Alice&#39;,
  &#39;type&#39;: &#39;Person&#39;},
 True)</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="k">for</span> <span class="n">k</span><span class="p">,</span> <span class="n">v</span> <span class="ow">in</span> <span class="n">person_item</span><span class="o">.</span><span class="vm">__dict__</span><span class="o">.</span><span class="n">items</span><span class="p">():</span>
    <span class="k">if</span> <span class="n">person_from_db</span><span class="o">.</span><span class="vm">__dict__</span><span class="p">[</span><span class="n">k</span><span class="p">]</span> <span class="o">!=</span> <span class="n">v</span><span class="p">:</span>
        <span class="nb">print</span><span class="p">(</span><span class="n">k</span><span class="p">)</span>
        <span class="nb">print</span><span class="p">(</span><span class="n">v</span><span class="p">)</span>
        <span class="nb">print</span><span class="p">(</span><span class="n">person_from_db</span><span class="o">.</span><span class="vm">__dict__</span><span class="p">[</span><span class="n">k</span><span class="p">])</span>
        <span class="nb">print</span><span class="p">()</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Appart from creating, we might want to update existing items:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item</span><span class="o">.</span><span class="n">lastName</span> <span class="o">=</span> <span class="s2">&quot;Awesome&quot;</span>
<span class="n">client</span><span class="o">.</span><span class="n">update_item</span><span class="p">(</span><span class="n">person_item</span><span class="p">)</span>
<span class="n">person_from_db</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">person_item</span><span class="o">.</span><span class="n">id</span><span class="p">,</span> <span class="n">expanded</span><span class="o">=</span><span class="kc">False</span><span class="p">)</span>
<span class="nb">print</span><span class="p">(</span><span class="n">person_from_db</span><span class="o">.</span><span class="n">lastName</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>Awesome
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Sync">Sync</h2><p>the <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a> can automatically sync your local items with the Pod with the <a href="/component-architectures/plugins/pod_client#PodClient.sync"><code>PodClient.sync</code></a> method. This method has a <code>priority</code> argument to resolve sync conflicts. The available strategies are:</p>
<ul>
<li><code>"local"</code>: Use the local value if a sync conflict is detected</li>
<li><code>"remote"</code>: Use the remote value if a sync conflict is detected</li>
<li><code>"newest"</code>: Use the last updated value if a sync conflict is detected. Warning: the pod only stores when a full item is updated. This strategy could lead to unexpected behaviour in cases where sync conflicts happen with a high frequency.</li>
<li><code>"error"</code>: Throw a <code>ValueError</code> when a sync conflict is detected.</li>
</ul>
<p>You can simply add new items to the sync with:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">account</span> <span class="o">=</span> <span class="n">Account</span><span class="p">(</span><span class="n">handle</span><span class="o">=</span><span class="s2">&quot;alice&quot;</span><span class="p">)</span>
<span class="n">person</span> <span class="o">=</span> <span class="n">Person</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Alice&quot;</span><span class="p">)</span>
<span class="n">account</span><span class="o">.</span><span class="n">store</span><span class="p">(</span><span class="n">client</span><span class="p">)</span>
<span class="n">person</span><span class="o">.</span><span class="n">store</span><span class="p">(</span><span class="n">client</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>Person (#e340f889884b4e4c91bc6711b38d9652)</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p><code>sync</code> also synchronizes edges if both the source and target item are stored for syncing:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">account</span><span class="o">.</span><span class="n">add_edge</span><span class="p">(</span><span class="s2">&quot;owner&quot;</span><span class="p">,</span> <span class="n">person</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">sync</span><span class="p">()</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>BULK: Writing 9/9 items/edges
Completed Bulk action, written 9 items/edges
</pre>
</div>
</div>
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>True</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span><span class="o">.</span><span class="n">reset_local_db</span><span class="p">()</span>
<span class="n">account</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">account</span><span class="o">.</span><span class="n">id</span><span class="p">)</span>
<span class="k">assert</span> <span class="n">account</span><span class="o">.</span><span class="n">owner</span><span class="p">[</span><span class="mi">0</span><span class="p">]</span><span class="o">.</span><span class="n">id</span> <span class="o">==</span> <span class="n">person</span><span class="o">.</span><span class="n">id</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">all_items</span> <span class="o">=</span> <span class="n">setup_sync_test</span><span class="p">(</span><span class="n">client</span><span class="p">)</span>
<span class="k">for</span> <span class="nb">id</span><span class="p">,</span> <span class="n">node</span> <span class="ow">in</span> <span class="n">client</span><span class="o">.</span><span class="n">local_db</span><span class="o">.</span><span class="n">nodes</span><span class="o">.</span><span class="n">items</span><span class="p">():</span>
    <span class="nb">print</span><span class="p">(</span><span class="nb">id</span><span class="p">,</span> <span class="n">node</span><span class="o">.</span><span class="n">_in_pod</span><span class="p">)</span>
    
<span class="k">for</span> <span class="n">item</span> <span class="ow">in</span> <span class="n">all_items</span><span class="p">:</span>
    <span class="nb">print</span><span class="p">(</span><span class="n">item</span><span class="o">.</span><span class="n">id</span><span class="p">,</span> <span class="n">item</span><span class="o">.</span><span class="n">_in_pod</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>BULK: Writing 3/3 items/edges
Completed Bulk action, written 3 items/edges
4c2913f892c8463ebdd2ee2a8197cb8e True
e340f889884b4e4c91bc6711b38d9652 True
5411889908c44b4bbebf469308065b66 True
a7d7e0687c36451bb105097e5c8bb304 True
eb4c84d002ac4ffebfae69a9a4597b5e True
a6f801efb04946b4885b9d62fb445a0f False
c8bae9ccff19410099d7e5ed8ae0e954 False
60e6e59dbf104d4b976f68fbbce8d325 False
5411889908c44b4bbebf469308065b66 True
a7d7e0687c36451bb105097e5c8bb304 True
eb4c84d002ac4ffebfae69a9a4597b5e True
a6f801efb04946b4885b9d62fb445a0f False
c8bae9ccff19410099d7e5ed8ae0e954 False
60e6e59dbf104d4b976f68fbbce8d325 False
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Search">Search</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>the <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a> can search through the pod with the <code>search</code> or <code>search_paginate</code> methods, which return the results of a search as a list or generator respectively. Search uses the same arguments as the Pod search API, which can be found <a href="https://gitlab.memri.io/memri/pod/-/blob/dev/docs/HTTP_API.md#post-v4owner_keysearch">here</a>.</p>
<p>To display how search works, we first add a few new items</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item2</span> <span class="o">=</span> <span class="n">Person</span><span class="o">.</span><span class="n">from_data</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Bob&quot;</span><span class="p">)</span>
<span class="n">person_account</span> <span class="o">=</span> <span class="n">Account</span><span class="p">(</span><span class="n">service</span><span class="o">=</span><span class="s2">&quot;testService&quot;</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">person_item2</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">person_account</span><span class="p">)</span>
<span class="n">person_item2</span><span class="o">.</span><span class="n">add_edge</span><span class="p">(</span><span class="s2">&quot;account&quot;</span><span class="p">,</span> <span class="n">person_account</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create_edges</span><span class="p">(</span><span class="n">person_item2</span><span class="o">.</span><span class="n">get_edges</span><span class="p">(</span><span class="s2">&quot;account&quot;</span><span class="p">));</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>BULK: Writing 1/1 items/edges
Completed Bulk action, written 1 items/edges
</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">all_people</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">search</span><span class="p">({</span><span class="s2">&quot;type&quot;</span><span class="p">:</span> <span class="s2">&quot;Person&quot;</span><span class="p">},</span> <span class="n">include_edges</span><span class="o">=</span><span class="kc">True</span><span class="p">)</span>
<span class="nb">print</span><span class="p">(</span><span class="s2">&quot;Number of results:&quot;</span><span class="p">,</span> <span class="nb">len</span><span class="p">(</span><span class="n">all_people</span><span class="p">))</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>Number of results: 5
</pre>
</div>
</div>
</div>
</div>
</div>
    
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">ids</span> <span class="o">=</span> <span class="p">[</span><span class="n">person</span><span class="o">.</span><span class="n">id</span> <span class="k">for</span> <span class="n">person</span> <span class="ow">in</span> <span class="n">all_people</span><span class="p">]</span>
<span class="n">result</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">search</span><span class="p">({</span><span class="s2">&quot;ids&quot;</span><span class="p">:</span> <span class="n">ids</span><span class="p">})</span>
<span class="k">assert</span> <span class="p">[</span><span class="n">item</span><span class="o">.</span><span class="n">id</span> <span class="k">for</span> <span class="n">item</span> <span class="ow">in</span> <span class="n">result</span><span class="p">]</span> <span class="o">==</span> <span class="n">ids</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>To hande large volumes of Items, the <a href="/component-architectures/plugins/pod_client#PodClient.search_paginate"><code>PodClient.search_paginate</code></a> method can search through the pod and return a generator which yields batches of items. This method uses the same search arguments as the <code>search</code> method:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">client</span><span class="o">.</span><span class="n">bulk_action</span><span class="p">(</span>
    <span class="n">create_items</span><span class="o">=</span><span class="p">[</span>
        <span class="n">Account</span><span class="p">(</span><span class="n">identifier</span><span class="o">=</span><span class="nb">str</span><span class="p">(</span><span class="n">i</span><span class="p">),</span> <span class="n">service</span><span class="o">=</span><span class="s2">&quot;paginate_test&quot;</span><span class="p">)</span> <span class="k">for</span> <span class="n">i</span> <span class="ow">in</span> <span class="nb">range</span><span class="p">(</span><span class="mi">100</span><span class="p">)</span>
    <span class="p">]</span>
<span class="p">)</span>
<span class="n">generator</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">search_paginate</span><span class="p">({</span><span class="s2">&quot;type&quot;</span><span class="p">:</span> <span class="s2">&quot;Account&quot;</span><span class="p">,</span> <span class="s2">&quot;service&quot;</span><span class="p">:</span> <span class="s2">&quot;paginate_test&quot;</span><span class="p">},</span> <span class="n">limit</span><span class="o">=</span><span class="mi">10</span><span class="p">)</span>
<span class="k">for</span> <span class="n">page</span> <span class="ow">in</span> <span class="n">generator</span><span class="p">:</span>
    <span class="c1"># process accounts</span>
    <span class="k">pass</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>BULK: Writing 100/100 items/edges
Completed Bulk action, written 100 items/edges
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Search-last-added-items">Search last added items</h2>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item2</span> <span class="o">=</span> <span class="n">Person</span><span class="o">.</span><span class="n">from_data</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Last Person&quot;</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">person_item2</span><span class="p">)</span>
<span class="n">last_added</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">search_last_added</span><span class="p">(</span><span class="nb">type</span><span class="o">=</span><span class="s2">&quot;Person&quot;</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>In the near future, Pod will support searching by user defined properties as well. This will allow for the following. <strong>warning, this is currently not supported</strong></p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p><code>client.search_last_added(type="Person", with_prop="ImportedBy", with_val="EmailImporter")</code></p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Uploading-&amp;-downloading-files">Uploading &amp; downloading files</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h3 id="File-API">File API</h3>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>To work with files like Photos or Videos, the <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a> has a separate file api. This api works by posting a blob to the <code>upload_file</code> endpoint, and creating an Item with a property with the same sha256 as the sha used in the endpoint.</p>
<p>For example, we can upload a photo with the file API as follows:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">x</span> <span class="o">=</span> <span class="n">np</span><span class="o">.</span><span class="n">random</span><span class="o">.</span><span class="n">randint</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="mi">255</span><span class="o">+</span><span class="mi">1</span><span class="p">,</span> <span class="n">size</span><span class="o">=</span><span class="p">(</span><span class="mi">640</span><span class="p">,</span> <span class="mi">640</span><span class="p">),</span> <span class="n">dtype</span><span class="o">=</span><span class="n">np</span><span class="o">.</span><span class="n">uint8</span><span class="p">)</span>
<span class="n">photo</span> <span class="o">=</span> <span class="n">Photo</span><span class="o">.</span><span class="n">from_np</span><span class="p">(</span><span class="n">x</span><span class="p">)</span>
<span class="n">file</span> <span class="o">=</span> <span class="n">photo</span><span class="o">.</span><span class="n">file</span><span class="p">[</span><span class="mi">0</span><span class="p">]</span>
<span class="n">succes</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">file</span><span class="p">)</span>
<span class="n">succes2</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">_upload_image</span><span class="p">(</span><span class="n">photo</span><span class="o">.</span><span class="n">data</span><span class="p">)</span>
<span class="n">cb</span> <span class="o">=</span> <span class="k">lambda</span> <span class="n">res</span><span class="p">:</span> <span class="nb">print</span><span class="p">(</span><span class="n">res</span><span class="p">)</span>
<span class="n">succes3</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">_upload_image</span><span class="p">(</span><span class="n">photo</span><span class="o">.</span><span class="n">data</span><span class="p">,</span> <span class="n">asyncFlag</span><span class="o">=</span><span class="kc">True</span><span class="p">,</span> <span class="n">callback</span><span class="o">=</span><span class="n">cb</span><span class="p">)</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>True
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h3 id="Photo-API">Photo API</h3>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>The PodClient implements an easier API for photos separately, which uses the same file API under the hood</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="nb">print</span><span class="p">(</span><span class="n">client</span><span class="o">.</span><span class="n">registered_classes</span><span class="p">[</span><span class="s2">&quot;Photo&quot;</span><span class="p">])</span>
<span class="c1"># client.add_to_schema(Photo)</span>
<span class="n">x</span> <span class="o">=</span> <span class="n">np</span><span class="o">.</span><span class="n">random</span><span class="o">.</span><span class="n">randint</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="mi">255</span><span class="o">+</span><span class="mi">1</span><span class="p">,</span> <span class="n">size</span><span class="o">=</span><span class="p">(</span><span class="mi">640</span><span class="p">,</span> <span class="mi">640</span><span class="p">),</span> <span class="n">dtype</span><span class="o">=</span><span class="n">np</span><span class="o">.</span><span class="n">uint8</span><span class="p">)</span>
<span class="n">photo</span> <span class="o">=</span> <span class="n">Photo</span><span class="o">.</span><span class="n">from_np</span><span class="p">(</span><span class="n">x</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create_photo</span><span class="p">(</span><span class="n">photo</span><span class="p">);</span>
<span class="n">photo</span><span class="o">.</span><span class="n">file</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>&lt;class &#39;pymemri.data.photo.Photo&#39;&gt;
BULK: Writing 3/3 items/edges
Completed Bulk action, written 3 items/edges
</pre>
</div>
</div>
<div class="output_area">

<div class="output_text output_subarea output_execute_result">
<pre>[File (#ac52d3a0a99f46499487750430649476)]</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Some photos come as bytes, for example when downloading them from a third party service. We can use <code>photo.from_bytes</code> to initialize these photos:</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">byte_photo</span> <span class="o">=</span> <span class="sa">b</span><span class="s1">&#39;</span><span class="se">\x89</span><span class="s1">PNG</span><span class="se">\r\n\x1a\n\x00\x00\x00\r</span><span class="s1">IHDR</span><span class="se">\x00\x00\x00\xe1\x00\x00\x00\xe1\x08\x03\x00\x00\x00\t</span><span class="s1">m&quot;H</span><span class="se">\x00\x00\x00</span><span class="s1">3PLTE</span><span class="se">\x04\x02\x04\x00\x00\x00\xa0\xa0\xa0\xa3\xa3\xa3\xaa\xaa\xaa\xb4\xb4\xb4\xbd\xbe\xbd\xbb\xbc\xbb\xde\xde\xde\x9b\x9a\x9b\xfe\xfe\xfe\xf2\xf3\xf2\xe5\xe6\xe5\xd8\xd9\xd8\xd1\xd1\xd1\xc9\xca\xc9\xae\xae\xae\x80</span><span class="s1">k</span><span class="se">\x98\xfc\x00\x00\x01</span><span class="s1">TIDATx</span><span class="se">\x9c\xed\xdd</span><span class="s1">;r</span><span class="se">\xc2</span><span class="s1">P</span><span class="se">\x00\x04</span><span class="s1">A!</span><span class="se">\x90\x84\xfc\x01\xee\x7f</span><span class="s1">Z</span><span class="se">\x13</span><span class="s1">8</span><span class="se">\xb1\x13</span><span class="s1">S</span><span class="se">\xce</span><span class="s1">F</span><span class="se">\xaf\xfb\x06\x93</span><span class="s1">o</span><span class="se">\xd5</span><span class="s1">No</span><span class="se">\xef\x1f\x9f\xb7\xfb</span><span class="s1">}]</span><span class="se">\xd7</span><span class="s1">my</span><span class="se">\xba</span><span class="s1">|;</span><span class="se">\xff</span><span class="s1">4</span><span class="se">\xff\xdf\xf9</span><span class="s1">O</span><span class="se">\x97</span><span class="s1">W&lt;</span><span class="se">\x96</span><span class="s1">W</span><span class="se">\xac\xbf</span><span class="s1">m</span><span class="se">\xd7</span><span class="s1">i9</span><span class="se">\x1d\xdb\xfe</span><span class="s1">,</span><span class="se">\x9c\x8e\xec</span><span class="s1">4+</span><span class="se">\xac</span><span class="s1">{</span><span class="se">\x16</span><span class="s1">^</span><span class="se">\x14\xb6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2\xbe</span><span class="s1">!</span><span class="se">\n\xcf\n\xdb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}C</span><span class="se">\x14\xce\n\xdb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8</span><span class="s1">7</span><span class="se">\xc4</span><span class="s1">bHa</span><span class="se">\x9c\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x86</span><span class="s1">xaQ</span><span class="se">\x18\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8</span><span class="s1">7D</span><span class="se">\xe1\xe3\xf0\x85\x8b\xc2</span><span class="s1">6</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}</span><span class="se">\n\xfb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}C</span><span class="se">\x14\xae\n\xdb\x14\xf6</span><span class="s1">)</span><span class="se">\xec</span><span class="s1">S</span><span class="se">\xd8\xa7\xb0</span><span class="s1">Oa</span><span class="se">\x9f\xc2</span><span class="s1">&gt;</span><span class="se">\x85</span><span class="s1">}C</span><span class="se">\x14</span><span class="s1">n</span><span class="se">\xa7</span><span class="s1">c</span><span class="se">\xdb\xa7\xeb</span><span class="s1">&gt;</span><span class="se">\x1f\xd9</span><span class="s1">~</span><span class="se">\xfb\x02\xee\x7f\r\xe5\xe1</span><span class="s1">h</span><span class="se">\x04</span><span class="s1">&quot;</span><span class="se">\x00\x00\x00\x00</span><span class="s1">IEND</span><span class="se">\xae</span><span class="s1">B`</span><span class="se">\x82</span><span class="s1">&#39;</span>
<span class="n">photo</span> <span class="o">=</span> <span class="n">Photo</span><span class="o">.</span><span class="n">from_bytes</span><span class="p">(</span><span class="n">byte_photo</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create_photo</span><span class="p">(</span><span class="n">photo</span><span class="p">);</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>BULK: Writing 3/3 items/edges
Completed Bulk action, written 3 items/edges
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Bulk-API">Bulk API</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Adding each item separately to the pod with the <code>create</code> method can take a lot of time. For this reason, using the bulk API is faster and more convenient in most cases. Here we show creating items and edges, updating and deleting is also possible.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">dogs</span> <span class="o">=</span> <span class="p">[</span><span class="n">Dog</span><span class="p">(</span><span class="n">name</span><span class="o">=</span><span class="sa">f</span><span class="s2">&quot;dog number </span><span class="si">{</span><span class="n">i</span><span class="si">}</span><span class="s2">&quot;</span><span class="p">)</span> <span class="k">for</span> <span class="n">i</span> <span class="ow">in</span> <span class="nb">range</span><span class="p">(</span><span class="mi">100</span><span class="p">)]</span>
<span class="n">person</span> <span class="o">=</span> <span class="n">Person</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Alice&quot;</span><span class="p">)</span>
<span class="n">edge1</span> <span class="o">=</span> <span class="n">Edge</span><span class="p">(</span><span class="n">dogs</span><span class="p">[</span><span class="mi">0</span><span class="p">],</span> <span class="n">person</span><span class="p">,</span> <span class="s2">&quot;label&quot;</span><span class="p">)</span>
<span class="n">edge2</span> <span class="o">=</span> <span class="n">Edge</span><span class="p">(</span><span class="n">dogs</span><span class="p">[</span><span class="mi">1</span><span class="p">],</span> <span class="n">person</span><span class="p">,</span> <span class="s2">&quot;label&quot;</span><span class="p">)</span>
<span class="c1"># Simultaneously add the dogs, person, and edges with the bulk API</span>
<span class="n">success</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">bulk_action</span><span class="p">(</span><span class="n">create_items</span><span class="o">=</span><span class="n">dogs</span> <span class="o">+</span> <span class="p">[</span><span class="n">person</span><span class="p">],</span> <span class="n">create_edges</span><span class="o">=</span><span class="p">[</span><span class="n">edge1</span><span class="p">,</span> <span class="n">edge2</span><span class="p">])</span>
</pre></div>
    </div>
</div>
</div>
<div class="output_wrapper">
<div class="output">
<div class="output_area">
<div class="output_subarea output_stream output_stdout output_text">
<pre>BULK: Writing 103/103 items/edges
Completed Bulk action, written 103 items/edges
</pre>
</div>
</div>
</div>
</div>
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h1 id="Create-items-that-do-not-exist-in-the-Pod">Create items that do not exist in the Pod</h1><p>The <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a> can deduplicate items with the same externalId with the <code>create_if_external_id_not_exists</code> method.</p>
</div>
</div>
</div>
    
    
<div class="cell border-box-sizing code_cell rendered">
<div class="input">
<div class="inner_cell">
    <div class="input_area">
<div class=" highlight hl-ipython3 chroma"><pre class="nbdev-code chroma"><span></span><span class="n">person_item</span> <span class="o">=</span> <span class="n">Person</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Eve&quot;</span><span class="p">,</span> <span class="n">externalId</span><span class="o">=</span><span class="s2">&quot;gmail_1&quot;</span><span class="p">)</span>
<span class="n">person_item2</span> <span class="o">=</span> <span class="n">Person</span><span class="p">(</span><span class="n">firstName</span><span class="o">=</span><span class="s2">&quot;Eve2&quot;</span><span class="p">,</span> <span class="n">externalId</span><span class="o">=</span><span class="s2">&quot;gmail_1&quot;</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create_if_external_id_not_exists</span><span class="p">(</span><span class="n">person_item</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create_if_external_id_not_exists</span><span class="p">(</span><span class="n">person_item2</span><span class="p">)</span>
<span class="n">existing</span> <span class="o">=</span> <span class="n">client</span><span class="o">.</span><span class="n">search</span><span class="p">({</span><span class="s2">&quot;externalId&quot;</span><span class="p">:</span> <span class="s2">&quot;gmail_1&quot;</span><span class="p">})</span>
</pre></div>
    </div>
</div>
</div>
</div>
    
</div>
 
