---
title: 'Readme'
date: date:2018-11-28T15:14:39+10:00
weight: 1
---
**Pymemri is a python library for creating <b>Plugins</b> for the Memri Personal online datastore <a href='https://gitlab.memri.io/memri/pod'>(pod)</a>. Pymemri has a PodClient to communicate with the pod, and tools to build and test plugins.**
<!--
#################################################
### THIS FILE WAS AUTOGENERATED! DO NOT EDIT! ###
#################################################
# file to edit: nbs/index.ipynb
# command to build the docs after a change: nbdev_build_docs
-->
<div class="container" id="notebook-container">
        
    
    
<div class="cell border-box-sizing code_cell rendered">
</div>
    
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p><a href="https://gitlab.memri.io/memri/pymemri/-/pipelines/latest"><img src="https://img.shields.io/gitlab/pipeline/memri/pymemri/dev?gitlab_url=https%3A%2F%2Fgitlab.memri.io&amp;label=CI&amp;logo=gitlab&amp;style=plastic" alt="Gitlab pipeline status (self-hosted)"></a>
<a href="https://discord.gg/BcRfajJk4k"><img src="https://img.shields.io/discord/799216875480678430?color=blue&amp;label=Discord&amp;logo=discord&amp;style=plastic" alt="Discord"></a>
<a href="https://twitter.com/YourMemri"><img src="https://img.shields.io/twitter/url?label=%40YourMemri&amp;logo=twitter&amp;style=plastic&amp;url=https%3A%2F%2Ftwitter.com%2FYourMemri" alt="Twitter URL"></a>
<a href="https://pypi.org/project/pymemri/"><img src="https://pepy.tech/badge/pymemri" /></a></p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>Plugins connect and add the information to your Pod. Plugins that <b>import your data from external services</b> are called <strong>Importers</strong> (Gmail, WhatsApp, etc.). Plugins that <b>connect new data to the existing data</b> are called  <strong>indexers</strong> (face recognition, spam detection, object detection, etc.). Lastly there are plugins that <b>execute actions</b> (sending messages, uploading files). This repository is built with <a href="https://github.com/fastai/nbdev">nbdev</a>, which means that the repo structure has a few differences compared to a standard python repo.</p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Installing">Installing</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h3 id="As-a-package">As a package</h3><div class="highlight"><pre class="nbdev-code chroma"><span></span>pip install pymemri
</pre></div>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h3 id="Development">Development</h3><p>To install the Python package, and correctly setup nbdev for development run:</p>
<div class="highlight"><pre class="nbdev-code chroma"><span></span>pip install -e . <span class="o">&amp;&amp;</span> nbdev_install_git_hooks
</pre></div>
<p>The last command configures git to automatically clean metadata from your notebooks before a commit.</p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Quickstart:-Pod-Client">Quickstart: Pod Client</h2><p>All interaction between plugins and the pod goes via the Pymemri <a href="/component-architectures/plugins/pod_client#PodClient"><code>PodClient</code></a>. To use this client in development, we first need to have a pod running locally. The quickest way to do this is to install from the <a href="https://gitlab.memri.io/memri/pod">pod repo</a>, and run <code>./examples/run_development.sh</code>.</p>
<p>If you have a running pod, you can define and add your own item definitions:</p>
<div class="highlight"><pre class="nbdev-code chroma"><span></span><span class="kn">from</span> <span class="nn">pymemri.data.itembase</span> <span class="kn">import</span> <span class="n">Item</span>
<span class="kn">from</span> <span class="nn">pymemri.pod.client</span> <span class="kn">import</span> <span class="n">PodClient</span>
<span class="k">class</span> <span class="nc">Dog</span><span class="p">(</span><span class="n">Item</span><span class="p">):</span>
    <span class="n">properties</span> <span class="o">=</span> <span class="n">Item</span><span class="o">.</span><span class="n">properties</span> <span class="o">+</span> <span class="p">[</span><span class="s2">&quot;name&quot;</span><span class="p">,</span> <span class="s2">&quot;age&quot;</span><span class="p">]</span>
    <span class="k">def</span> <span class="fm">__init__</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">name</span><span class="p">:</span> <span class="nb">str</span> <span class="o">=</span> <span class="kc">None</span><span class="p">,</span> <span class="n">age</span><span class="p">:</span> <span class="nb">int</span> <span class="o">=</span> <span class="kc">None</span><span class="p">,</span> <span class="o">**</span><span class="n">kwargs</span><span class="p">):</span>
        <span class="nb">super</span><span class="p">()</span><span class="o">.</span><span class="fm">__init__</span><span class="p">(</span><span class="o">**</span><span class="n">kwargs</span><span class="p">)</span>
        <span class="bp">self</span><span class="o">.</span><span class="n">name</span> <span class="o">=</span> <span class="n">name</span>
        <span class="bp">self</span><span class="o">.</span><span class="n">age</span> <span class="o">=</span> <span class="n">age</span>
<span class="c1"># Connect to the pod and add the Dog item definition</span>
<span class="n">client</span> <span class="o">=</span> <span class="n">PodClient</span><span class="p">()</span>
<span class="n">client</span><span class="o">.</span><span class="n">add_to_schema</span><span class="p">(</span><span class="n">Dog</span><span class="p">)</span>
<span class="c1"># Add a Dog to the pod</span>
<span class="n">dog</span> <span class="o">=</span> <span class="n">Dog</span><span class="p">(</span><span class="s2">&quot;bob&quot;</span><span class="p">,</span> <span class="mi">3</span><span class="p">)</span>
<span class="n">client</span><span class="o">.</span><span class="n">create</span><span class="p">(</span><span class="n">dog</span><span class="p">)</span>
</pre></div>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Quickstart:-Running-a-plugin">Quickstart: Running a plugin</h2>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>After installation, users can use the plugin CLI to manually run a plugin. For more information on how to build a plugin, see <a href="/component-architectures/plugins/pluginbase#run_plugin"><code>run_plugin</code></a>.</p>
<p><b>With the pod running, run in your terminal|: </b></p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<div class="highlight"><pre class="nbdev-code chroma"><span></span>store_keys
run_plugin --metadata <span class="s2">&quot;example_plugin.json&quot;</span>
</pre></div>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<p>This stores a random owner key and database key on your disk for future use, and runs the pymemri example plugin. If everything works correctly, the output should read <code>Plugin run success.</code></p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Docs">Docs</h2><p><a href="https://memri.docs.memri.io/docs.memri.io/component-architectures/plugins/readme/">pymemri docs</a></p>
</div>
</div>
</div>
<div class="cell border-box-sizing text_cell rendered"><div class="inner_cell">
<div class="text_cell_render border-box-sizing rendered_html">
<h2 id="Nbdev-&amp;-Jupyter-Notebooks">Nbdev &amp; Jupyter Notebooks</h2><p>The Python integrators are written in <a href="https://nbdev.fast.ai/">nbdev</a> (<a href="https://www.youtube.com/watch?v=9Q6sLbz37gk&amp;t=1301s">video</a>). With nbdev, it is encouraged to write code in 
<a href="https://jupyter.readthedocs.io/en/latest/install/notebook-classic.html">Jupyter Notebooks</a>. Nbdev syncs all the notebooks in <code>/nbs</code> with the python code in <code>/pymemri</code>. Tests are written side by side with the code in the notebooks, and documentation is automatically generated from the code and markdown in the notebooks and exported into the <code>/docs</code> folder. Check out the <a href="wiki/nbdev_quickstart.md">nbdev quickstart</a> for an introduction, <strong>watch the video linked above</strong>, or see the <a href="https://nbdev.fast.ai/">nbdev documentation</a> for a all functionalities and tutorials.</p>
</div>
</div>
</div>
</div>
 
